@@ -1,4 +1,5 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  * Copyright (C) 2005 Vincent Untz
  *
  * This program is free software; you can redistribute it and/or
@@ -141,7 +142,8 @@ activate_desktop_uri (GtkWidget *menuitem,
 static void
 panel_menu_items_append_from_desktop (GtkWidget *menu,
 				      char      *path,
-				      char      *force_name)
+				      char      *force_name,
+                                      gboolean   use_icon)
 {
 	GKeyFile  *key_file;
 	gboolean   loaded;
@@ -237,7 +239,12 @@ panel_menu_items_append_from_desktop (GtkWidget *menu,
 	else
 		name = g_strdup (force_name);
 
-	item = gtk_image_menu_item_new ();
+	if (use_icon) {
+		item = panel_image_menu_item_new ();
+        } else {
+		item = gtk_image_menu_item_new ();
+	}
+
 	setup_menu_item_with_icon (item, panel_menu_icon_get_size (),
 				   icon, NULL, NULL, name);
 
@@ -283,7 +290,7 @@ panel_menu_items_append_place_item (const char *icon_name,
 	GtkWidget *item;
 	char      *user_data;
 
-	item = gtk_image_menu_item_new ();
+	item = panel_image_menu_item_new ();
 	setup_menu_item_with_icon (item,
 				   panel_menu_icon_get_size (),
 				   icon_name, NULL, gicon,
@@ -555,7 +562,7 @@ panel_menu_item_append_drive (GtkWidget *menu,
 	icon = g_drive_get_icon (drive);
 	title = g_drive_get_name (drive);
 
-	item = gtk_image_menu_item_new ();
+	item = panel_image_menu_item_new ();
 	setup_menu_item_with_icon (item,
 				   panel_menu_icon_get_size (),
 				   NULL, NULL, icon,
@@ -655,7 +662,7 @@ panel_menu_item_append_volume (GtkWidget *menu,
 	icon = g_volume_get_icon (volume);
 	title = g_volume_get_name (volume);
 
-	item = gtk_image_menu_item_new ();
+	item = panel_image_menu_item_new ();
 	setup_menu_item_with_icon (item,
 				   panel_menu_icon_get_size (),
 				   NULL, NULL, icon,
@@ -831,6 +838,11 @@ panel_place_menu_item_append_local_gio (PanelPlaceMenuItem *place_item,
 
 		mount = l->data;
 
+		if (g_mount_is_shadowed (mount)) {
+			g_object_unref (mount);
+			continue;
+		}
+
 		volume = g_mount_get_volume (mount);
 		if (volume != NULL) {
 			g_object_unref (volume);
@@ -917,6 +929,12 @@ panel_place_menu_item_append_remote_gio (PanelPlaceMenuItem *place_item,
 		GFile   *root;
 
 		mount = l->data;
+
+		if (g_mount_is_shadowed (mount)) {
+			g_object_unref (mount);
+			continue;
+		}
+
 		volume = g_mount_get_volume (mount);
 		if (volume != NULL) {
 			g_object_unref (volume);
@@ -942,7 +960,7 @@ panel_place_menu_item_append_remote_gio (PanelPlaceMenuItem *place_item,
 	} else {
 		GtkWidget  *item;
 
-		item = gtk_image_menu_item_new ();
+		item = panel_image_menu_item_new ();
 		setup_menu_item_with_icon (item, panel_menu_icon_get_size (),
 					   PANEL_ICON_NETWORK_SERVER,
 					   NULL, NULL,
@@ -1021,7 +1039,8 @@ panel_place_menu_item_create_menu (PanelPlaceMenuItem *place_item)
 					      NULL);
 	panel_menu_items_append_from_desktop (places_menu,
 					      "nautilus-computer.desktop",
-					      gconf_name);
+					      gconf_name,
+                                              TRUE);
 	if (gconf_name)
 		g_free (gconf_name);
 
@@ -1030,7 +1049,8 @@ panel_place_menu_item_create_menu (PanelPlaceMenuItem *place_item)
 
 	panel_menu_items_append_from_desktop (places_menu,
 					      "network-scheme.desktop",
-					      NULL);
+					      NULL,
+                                              TRUE);
 	panel_place_menu_item_append_remote_gio (place_item, places_menu);
 
 	if (panel_is_program_in_path ("nautilus-connect-server")) {
@@ -1044,7 +1064,8 @@ panel_place_menu_item_create_menu (PanelPlaceMenuItem *place_item)
 
 	panel_menu_items_append_from_desktop (places_menu,
 					      "gnome-search-tool.desktop",
-					      NULL);
+					      NULL,
+                                              FALSE);
 
 	panel_recent_append_documents_menu (places_menu,
 					    place_item->priv->recent_manager);
@@ -1136,8 +1157,8 @@ panel_desktop_menu_item_append_menu (GtkWidget *menu,
 	if (add_separator)
 		add_menu_separator (menu);
 
-	panel_menu_items_append_from_desktop (menu, "yelp.desktop", NULL);
-	panel_menu_items_append_from_desktop (menu, "gnome-about.desktop", NULL);
+	panel_menu_items_append_from_desktop (menu, "yelp.desktop", NULL, FALSE);
+	panel_menu_items_append_from_desktop (menu, "gnome-about.desktop", NULL, FALSE);
 
 	if (parent->priv->append_lock_logout)
 		panel_menu_items_append_lock_logout (menu);
@@ -1148,7 +1169,7 @@ panel_desktop_menu_item_create_menu (PanelDesktopMenuItem *desktop_item)
 {
 	GtkWidget *desktop_menu;
 
-	desktop_menu = create_applications_menu ("settings.menu", NULL);
+	desktop_menu = create_applications_menu ("settings.menu", NULL, FALSE);
 
 	g_object_set_data (G_OBJECT (desktop_menu),
 			   "panel-menu-append-callback",
